@@ -7,11 +7,12 @@ from collections import defaultdict
 from importlib import import_module
 from types import ModuleType
 
+from asgiref.local import Local
 from content_editor.models import Type
 from django.conf import settings
-from django.core.cache import cache
 from django.core.checks import Warning
 from django.core.exceptions import ValidationError
+from django.core.signals import request_finished
 from django.db import models
 from django.db.models import Q, signals
 from django.urls import NoReverseMatch, include, re_path, reverse
@@ -143,14 +144,18 @@ def reverse_fallback(fallback, fn, *args, **kwargs):
         return fallback
 
 
-_APPS_URLCONF_CACHE_KEY = "_apps_urlconf_cache"
+def _del_apps_urlconf_cache(**kwargs):
+    try:
+        del _apps_urlconf_cache.cache
+    except AttributeError:
+        pass
 
 
-def clear_apps_urlconf_cache(sender, **kwargs):
-    cache.delete(_APPS_URLCONF_CACHE_KEY)
+_apps_urlconf_cache = Local()
+request_finished.connect(_del_apps_urlconf_cache)
 
 
-def apps_urlconf(*, apps=None, timeout=3):
+def apps_urlconf(*, apps=None):
     """
     Generates a dynamic URLconf Python module including all application page
     types in their assigned place and adding the ``urlpatterns`` from
@@ -182,8 +187,7 @@ def apps_urlconf(*, apps=None, timeout=3):
     """
 
     if apps is None:
-        if timeout:
-            apps = cache.get(_APPS_URLCONF_CACHE_KEY)
+        apps = getattr(_apps_urlconf_cache, "cache", None)
 
         if apps is None:
             fields = ("path", "page_type", "app_namespace", "language_code")
@@ -197,7 +201,7 @@ def apps_urlconf(*, apps=None, timeout=3):
             # NOTE! We *could* cache the module_name instead but we'd still
             # have to check if the module actually exists in the local Python
             # process.
-            cache.set(_APPS_URLCONF_CACHE_KEY, apps, timeout=timeout)
+            _apps_urlconf_cache.cache = apps
 
     if not apps:
         # No point wrapping ROOT_URLCONF if there are no additional URLs
@@ -480,9 +484,6 @@ class PageTypeMixin(models.Model):
             global _APPS_MODEL
             _APPS_MODEL = sender
 
-            signals.post_save.connect(clear_apps_urlconf_cache, sender=sender)
-            signals.post_delete.connect(clear_apps_urlconf_cache, sender=sender)
-
     @classmethod
     def check(cls, **kwargs):
         errors = super().check(**kwargs)
